#define G_LOG_DOMAIN "boilerplate-preferences-group"

#include "config.h"

#include "boilerplate-preferences-group.h"

#include <gtk/gtk.h>

/**
 * SECTION:boilerplate-preferences-group
 * @title: BoilerplatePreferencesGroup
 * @short_description: A #GtkBin instance created for #BoilerplatePreferences
 * @include: boilerplate-preferences-group.h
 * @see_also: #GtkBin, #BoilerplatePreferencesGroup
 *
 * The #BoilerplatePreferencesGroup is an instance #GtkBin and is the group
 * preferences will be dumped into in the Preferences window.
 **/

typedef struct _BoilerplatePreferencesGroupPrivate
{
    GtkWidget *list_box;

    gchar *name;
    gchar *title;
} BoilerplatePreferencesGroupPrivate;

struct _BoilerplatePreferencesGroup
{
    GtkBin parent_instance;
};

G_DEFINE_TYPE_WITH_PRIVATE (BoilerplatePreferencesGroup, boilerplate_preferences_group, GTK_TYPE_BIN)

enum {
    PROP_0,
    PROP_NAME,
    PROP_TITLE,
    LAST_PROP
};

/**
 * boilerplate_preferences_group_add_item
 * @self: An instance of #BoilerplatePreferencesGroup
 * @child: The child #GtkWidget to add to the row.
 *
 * Adds a item (#GtkListBoxRow) to the #BoilerplatePreferencesGroup.
 */
static void
boilerplate_preferences_group_add (BoilerplatePreferencesGroup *self,
                                   GtkWidget                   *child)
{
    g_return_if_fail (BOILERPLATE_IS_PREFERENCES_GROUP (self));
    g_return_if_fail (GTK_IS_WIDGET (child));

    GtkWidget *group = boilerplate_preferences_group_get_list_box (self);

    gtk_container_add (GTK_CONTAINER (group), GTK_WIDGET (child));
}

/**
 * boilerplate_preferences_group_get_box:
 * @self: An instance of #BoilerplatePreferencesGroup
 *
 * Returns: The group's content area as a #GtkWidget which has an ID of "box".
 */
GtkWidget *
boilerplate_preferences_group_get_list_box (BoilerplatePreferencesGroup *self)
{
    g_return_val_if_fail (BOILERPLATE_IS_PREFERENCES_GROUP (self), NULL);

    BoilerplatePreferencesGroupPrivate *priv = boilerplate_preferences_group_get_instance_private (self);

    return GTK_WIDGET (priv->list_box);
}

/**
 * boilerplate_preferences_group_get_property:
 * @object: The current GObject
 * @prop_id: The property id
 * @value: The value to get
 * @pspec: A g_param_spec_* ()
 *
 * Get property in #BoilerplatePreferencesGroup.
 */
static void
boilerplate_preferences_group_get_property (GObject    *object,
                                            guint       prop_id,
                                            GValue     *value,
                                            GParamSpec *pspec)
{
    BoilerplatePreferencesGroup *self = BOILERPLATE_PREFERENCES_GROUP (object);
    BoilerplatePreferencesGroupPrivate *priv = boilerplate_preferences_group_get_instance_private (self);

    switch (prop_id)
    {
        case PROP_NAME:
            g_value_set_object (value,
                                priv->name);
            break;
        case PROP_TITLE:
            g_value_set_object (value,
                                priv->title);
            break;

        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
    }
}

/**
 * boilerplate_preferences_group_set_property:
 * @object: The current GObject
 * @prop_id: The property id
 * @value: The value to set
 * @pspec: A g_param_spec_* ()
 *
 * Set property in #BoilerplatePreferencesGroup.
 */
static void
boilerplate_preferences_group_set_property (GObject      *object,
                                            guint         prop_id,
                                            const GValue *value,
                                            GParamSpec   *pspec)
{
    BoilerplatePreferencesGroup *self = BOILERPLATE_PREFERENCES_GROUP (object);
    BoilerplatePreferencesGroupPrivate *priv = boilerplate_preferences_group_get_instance_private (self);

    switch (prop_id)
    {
        case PROP_NAME:
            //g_value_get_string (value)
            break;

        case PROP_TITLE:

            break;

        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
    }
}

/*
 * boilerplate_preferences_group_finalize:
 * @object: An instance of #BoilerplatePreferencesGroup
 *
 * Does any last minute clean-ups when the class ends.
 */
static void
boilerplate_preferences_group_finalize (GObject *object)
{
    G_OBJECT_CLASS (boilerplate_preferences_group_parent_class)->finalize (object);
}

/*
 * boilerplate_preferences_group_init:
 * @self: An instance of #BoilerplatePreferencesGroup
 *
 * Initializes various aspects of the window. Anything the window would need
 * before becoming presented should be done here.
 */
static void
boilerplate_preferences_group_init (BoilerplatePreferencesGroup *self)
{
    gtk_widget_init_template (GTK_WIDGET (self));
}

/*
 * boilerplate_preferences_group_class_init:
 * @klass: An instance of #BoilerplatePreferencesGroupClass
 *
 * Initializes the various virtural functions in #BoilerplatePreferencesGroupClass.
 * In addtition it connects the various #GtkWidgets from the gresource file
 * using gtk_widget_class_set_template_from_resource().
 */
static void
boilerplate_preferences_group_class_init (BoilerplatePreferencesGroupClass *klass)
{
    GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
    GObjectClass *object_class = G_OBJECT_CLASS (klass);

    object_class->finalize = boilerplate_preferences_group_finalize;

    object_class->get_property = boilerplate_preferences_group_get_property;
    object_class->set_property = boilerplate_preferences_group_set_property;

	/**
	 * BoilerplatePrefernecesGroup:name:
	 *
	 * ID name for group.
	 */
    g_object_class_install_property (object_class,
                                     PROP_NAME,
                                     g_param_spec_string ("name",
                                                          "Group name",
                                                          "Group name for back-end.",
                                                          NULL,
                                                          G_PARAM_READWRITE));

	/**
	 * BoilerplatePrefernecesGroup:title:
	 *
	 * Title name that will be seen above group.
	 */
    g_object_class_install_property (object_class,
                                     PROP_TITLE,
                                     g_param_spec_string ("title",
                                                          "Title name",
                                                          "Title name that will be above the group.",
                                                          NULL,
                                                          G_PARAM_READWRITE));

    gtk_widget_class_set_template_from_resource (widget_class,
                                                 PACKAGE_RESOURCES "/ui/preferences/group.ui");

    gtk_widget_class_bind_template_child_private (widget_class, BoilerplatePreferencesGroup, list_box);
}

/**
 * boilerplate_preferences_group_new:
 * @window: A #GtkApplicationWindow instance
 *
 * Creates a new #BoilerplatePreferencesGroup, a sub-object of #GtkBin.
 *
 * ## Usage
 * |[
 * BoilerplatePreferencesGroup *group;
 *
 * group = boilerplate_preferences_group_new ("general",
 *                                            "General");
 * ]|
 *
 * Returns: #BoilerplatePreferencesGroup; a sub-object of #GtkBin
 */
BoilerplatePreferencesGroup *
boilerplate_preferences_group_new (const gchar *name,
                                   const gchar *title)
{
    return g_object_new (BOILERPLATE_TYPE_PREFERENCES_GROUP,
                         NULL);
}
